/* lmperror.c
 *
 * Copyright (C) 1999-2000 Bernhard Pietsch <bjtp@gmx.net>
 *
 * This file is part of LMP library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdarg.h>
#include "lmp.h"

/* stop lmp with a message */
void lmp_stop (msg)
const char * msg; /* stop message */
{
  switch (lmp_proc_error) {
  case LMP_NO_ERROR :
    return;
  case LMP_UNKNOWN_OPERATOR:
    fprintf (stderr, "Unknown operator at argument %d: %s\n", lmp_token_args (), msg);
    exit (1);
  default:
    fprintf (stderr, "Internal error!\n");
    exit (1);
  }
}

/* stop because of an internal lmp error */
void lmp_error (const char *msg, ...) {
  va_list args;

  va_start (args, msg);
  vfprintf (stderr, msg, args);

  exit (1);
}

/* output a warning */
void lmp_warning (const char *msg, ...)
{
  va_list args;

  va_start (args, msg);
  vfprintf (stderr, msg, args);

  va_end (args);
}
