/* lmptoken.c
 *
 * Copyright (C) 1999-2000 Bernhard Pietsch <bjtp@gmx.net>
 *
 * This file is part of LMP library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "lmp.h"
#include "lmpinput.h"
#include "lmpmath.h"
#include <stdio.h>
#include <malloc.h>

/* vector of arguments for lmp */
char ** lmp_args = NULL;
/* index for lmp_args */
int lmp_argnum = 1;

/* scans the input for the next token
 * returns next token
 */
char * lmp_token_next () {
  static char arg [LMP_ARG_MAX_LENGTH];

  if (*lmp_args == NULL)
    return NULL;

  strcpy (arg, *lmp_args++);
  lmp_argnum++;
  return arg;
}

/* Increments the token pointer.
 * It does not return a token.
 */
int lmp_token_inc () {
  if (*lmp_args++ == NULL)
    return LMP_MISSING_ARG;
  else
    lmp_argnum++;

  return 0;
}

/* Get the number of the current argument
 */
int lmp_token_args () {
  return lmp_argnum;
}

int lmp_token_is_last () {
  if (*lmp_args == NULL)
    return 1;
  else
    return 0;
}

/* Compares a string with the next token
 * It compares the current token with "s", but it does not
 * increment the token pointer.
 * returns 1 if equal and 0 if not.
 */
int lmp_token_following_equals (s)
char * s; /* to be compared string */
{
  if (*lmp_args == NULL)
    return 0;
  else if (!strcmp (s, lmp_token_following ()))
    return 1;
  else
    return 0;
}

/* watch the following token
 * Does not increment the token pointer but returns
 * the following token.
 */
char * lmp_token_following () {
  static char arg [LMP_ARG_MAX_LENGTH];

  if (*lmp_args == NULL)
    return NULL;

  strcpy (arg, *lmp_args);
  return arg;
}

/* Returns the last token.
 */
char * lmp_token_prev () {
  static char arg [LMP_ARG_MAX_LENGTH];

  if (*(lmp_args-1) == NULL)
    return NULL;

  strcpy (arg, *(lmp_args-1));
  return arg;
}

int lmp_token_next_is_number () {
  char *s = lmp_token_following (), *endptr;
  lmp_value_type re;

  if (!s)
    lmp_error ("At argument %d value missed.\n", lmp_token_args ());

  if (lmp_vt_lookup (s, &re)) {
    re = STRTOFVAL (s, &endptr);

    if (*endptr != 0 && re == 0)
      return 0;
  }

  return 1;
}

/* Return a number
 * Reads a token. It expects a number. If the token is something else,
 * lmp will stop.
 * returns the value of the number.
 */
lmp_value_type lmp_token_next_number () {
  char *s = lmp_token_next (), *endptr;
  lmp_value_type re;

  if (!s)
    lmp_error ("At argument %d value missed.\n", lmp_token_args ());

  if (lmp_vt_lookup (s, &re)) {
    re = (lmp_value_type) STRTOFVAL (s, &endptr);

    if (*endptr != 0 && re == 0) {
      // Check if complex

      lmp_error ("At argument %d value expected.\n", lmp_token_args () - 1);
    }
    else if (*endptr != '\0') {
      lmp_warning ("Warning: Argument %d is no number.\n", lmp_token_args () - 1);
    }
  }

  return re;
}

#ifdef HAVE_COMPLEX
lmp_value_type lmp_token_next_complex () {
  lmp_value_type re;

  re = lmp_token_next_number ();
  if (lmp_token_following_equals (","))
    lmp_token_inc ();
  else
    lmp_error ("\",\" expected at argument %d\n", lmp_token_args ());

  re += lmp_token_next_number () * _Complex_I;

  return re;
}
#else
lmp_value_type lmp_token_next_complex () {
  lmp_error ("At argument %d: This binary does not support complex values.\n", lmp_token_args ());

  return 0;
}
#endif
